from pydoc import text
import dash
import json
import random
import dash_table

import dash_html_components as html
import dash_core_components as dcc
import pandas as pd
import plotly.figure_factory as ff
import plotly.graph_objs as go

from operator import add
from dash.dependencies import Input, Output
from wordview.text_analysis.visual_analysis import do_analysis, plotly_wordcloud, generate_label_plots
from wordview.mwes import MWE


# Process data
input_text_name = "IMDB Corpus"
imdb_train = pd.read_csv('data/imdb_train_sample.tsv', sep='\t', names=['label', 'text'])
imdb_train = imdb_train.sample(1000)

# Text
analysis_res = do_analysis(df=imdb_train,
                               out_dir='output_dir',
                               text_col='text',
                               label_cols=[('label', 'categorical')])

dist_setup = {
              'paper_bgcolor': '#007A78',
            }
doc_len_dist = ff.create_distplot([analysis_res.doc_lengths], group_labels=["distplot"], colors=["blue"])
doc_len_dist.update_layout(dist_setup)

fig_w_freq = go.Figure()
fig_w_freq.add_trace(go.Scattergl(x=analysis_res.zipf_x, y=analysis_res.zipf_y_emp, mode='markers'))
fig_w_freq.add_trace(go.Scattergl(x=analysis_res.zipf_x, y=analysis_res.zipf_y_theory, mode='markers'))
fig_w_freq.update_layout(dist_setup)

fig_noun_cloud = go.Figure(plotly_wordcloud(token_count_dic=analysis_res.nns))
fig_verb_cloud = go.Figure(plotly_wordcloud(token_count_dic=analysis_res.vs))
fig_adj_cloud = go.Figure(plotly_wordcloud(token_count_dic=analysis_res.jjs))

word_cloud_setup = {'plot_bgcolor': 'rgba(0, 0, 0, 0)',
                    'paper_bgcolor': 'rgba(0, 0, 0, 0)',
                    'xaxis_showgrid':False,
                    'yaxis_showgrid':False,
                    'xaxis_zeroline':False,
                    'yaxis_zeroline':False,
                    'yaxis_visible':False,
                    'yaxis_showticklabels':False,
                    'xaxis_visible':False,
                    'xaxis_showticklabels':False,
                    }

fig_noun_cloud.update_layout(word_cloud_setup)
fig_adj_cloud.update_layout(word_cloud_setup)
fig_verb_cloud.update_layout(word_cloud_setup)

# MWEs
mwe = MWE(df=imdb_train, mwe_types=["NC", "JNC"], text_column='text')
mwe.build_counts()
mwe.extract_mwes()
with open(mwe.mwe_file, "r") as file:
    mwes_dict = json.load(file)

def mwe_dict_to_tabledata(mwe_dict, mwe_type, max_n=100):
    # ncs = []
    # ams = []
    tabledata = []
    for k,v in mwe_dict[mwe_type].items():
        if len(tabledata) <= max_n:
            if v > 0:
                # ncs.append(k)
                # ams.append(v)
                tabledata.append({'MWE': k, 'AM':v})
            else:
                break
        else:
            break
    return tabledata

tabledata_ncs = mwe_dict_to_tabledata(mwe_dict=mwes_dict, mwe_type='NC')
tabledata_jnc = mwe_dict_to_tabledata(mwe_dict=mwes_dict, mwe_type='JNC')


# Bubble 
# ams = [am * 20 for am in ams]
# y = list(map(add, ams, [random.randrange(-int(max(ams)), int(max(ams))) for i in range(len(ams))]))
# fig_mwe = go.Figure(data=[
#             go.Scatter(
#                 x=ams,
#                 y=y,
#                 text=ncs,
#                 mode='markers+text',
#                 hoverinfo='text',
#                 # marker_size=ams,
#                 marker=dict(color='cyan', size=ams)
#         )
#     ]
# )

# Labels
labels_figure = generate_label_plots(df=imdb_train, label_cols=[('label', 'categorical'), ('label', 'categorical'), ('label', 'categorical'), ('label', 'categorical')])

app = dash.Dash(__name__, meta_tags=[{"name": "viewport", "content": "width=device-width"}])

app.layout = html.Div([
    # First row: Header
    html.Div([
        html.Div([
            html.Img(src=app.get_asset_url('random-logo.png'), id='logo', 
            style={'height':'150px', 'width':'auto', 'margin-bottom':'25px'})
        ], className='one-third column'),
        html.Div([
            html.Div([
                html.H3('SNLP', style={'margin-bottom': '0px', 'color': 'white'}),
                html.H5(f'Statistics & Insights from {input_text_name}', style={'margin-bottom': '0px', 'color': 'white'})
            ])
        ], className='one-half column', id = 'title'),
        html.Div([
            html.H6('Last Updated: 00:01 (UTC)',
                    style={'color': 'orange'})

        ], className='one-third column', id = 'title1')

    ], id='header', className='row flex-display', style={'margin-bottom': '25px'}),
    # Second row: Tabs
    html.Div([
        dcc.Tabs(
            id="tabs",
            value='tab-1',  # Default
            parent_className='custom-tabs',
            className='custom-tabs-container',
            children=[
                dcc.Tab(
                    label='Stats',
                    value='tab-1',
                    className='custom-tab',
                    selected_className='custom-tab--selected',
                    style= {'backgroundColor': '#050c22'},
                    selected_style= {'backgroundColor': '#119DFF'},
                ),
                dcc.Tab(
                    label='Part of Speech',
                    value='tab-2',
                    className='custom-tab',
                    selected_className='custom-tab--selected',
                    style= {'backgroundColor': '#050c22'},
                    selected_style= {'backgroundColor': '#119DFF'},
                ),
                dcc.Tab(
                    label='MWEs',
                    value='tab-3',
                    className='custom-tab',
                    selected_className='custom-tab--selected',
                    style= {'backgroundColor': '#050c22'},
                    selected_style= {'backgroundColor': '#119DFF'},
                ),
                dcc.Tab(
                    label='Labels',
                    value='tab-4',
                    className='custom-tab',
                    selected_className='custom-tab--selected',
                    style= {'backgroundColor': '#050c22'},
                    selected_style= {'backgroundColor': '#119DFF'},
                ),
                dcc.Tab(
                    label='Toolkit',
                    value='tab-5', 
                    className='custom-tab',
                    selected_className='custom-tab--selected',
                    style= {'backgroundColor': '#050c22'},
                    selected_style= {'backgroundColor': '#119DFF'},
                ),
            ]),
        html.Div(id='tabs-content')
    ])
], id = 'mainContainer', style={'display': 'flex', 'flex-direction': 'column'})


@app.callback(Output('tabs-content', 'children'),
              [Input('tabs', 'value')]
              )
def render_content(tab):
    if tab == 'tab-1':
        return html.Div([
            html.H4(style={'margin-top': '48px'}),
            # Cards
            html.Div([
                html.Div([
                    html.H6(children='Detected Language/s',
                            style={'textAlign': 'center',
                                'color': 'white'}),
                    html.P(", ".join(analysis_res.languages).strip(),
                            style={'textAlign': 'center',
                                'color': 'orange',
                                'fontSize': 40}),
                    ], className='card_container three columns'),
                html.Div([
                    html.H6(children='Number of Words',
                            style={'textAlign': 'center',
                                'color': 'white'}),
                    html.P(f"Unique Words: {analysis_res.type_count:,}",
                            style={'textAlign': 'center',
                                'color': 'orange',
                                'fontSize': 20}),
                    html.P(f"All Words: {analysis_res.token_count:,}",
                            style={'textAlign': 'center',
                                'color': 'orange',
                                'fontSize': 20}),
                    ], className='card_container three columns'),
                html.Div([
                    html.H6(children='Number of Documents',
                            style={'textAlign': 'center',
                                'color': 'white'}),
                    html.P(f"{analysis_res.doc_count}",
                            style={'textAlign': 'center',
                                'color': 'orange',
                                'fontSize': 40}),
                    ], className='card_container three columns'),
                html.Div([
                    html.H6(children='Median Document Length',
                            style={'textAlign': 'center',
                                'color': 'white'}),
                    html.P(f"{int(analysis_res.median_doc_len)}",
                            style={'textAlign': 'center',
                                'color': 'orange',
                                'fontSize': 40}),
                    ], className='card_container three columns'),
                    
            ], className='row flex display'),
            # Dropdown
            html.Div([
                    html.P('Select Distribution', className = 'fix_label', style = {'color': 'grey', 'margin-top': '2px'}),
                    dcc.Dropdown(id = 'select_option',
                                 multi = False,
                                 clearable = True,
                                 disabled = False,
                                 style = {'display': True},
                                 value = 'Document Length',
                                 placeholder = 'Select Distribution',
                                 options=[{'label': 'Word Frequency', 'value': 'wf'},
                                          {'label': 'Document Length', 'value': 'dl'},
                                         ],
                                        className = 'dcc_compon')],
                                className='dropdown', style={'width': '50%', 'float': 'left'}),
            html.Div(id='dd-content', style={'clear': 'both'}),
        ])
    elif tab == 'tab-2':
        return html.Div([
            # Cards
            html.Div([
                html.Div([
                    html.H6(children='Verbs',
                            style={'textAlign': 'center',
                                'color': 'white'}),
                    html.P(1000,
                            style={'textAlign': 'center',
                                'color': 'orange',
                                'fontSize': 40}),
                    ], className='card_container', style={'width': '16%', 'float': 'left'}),
                html.Div([
                    html.H6(children='Nouns',
                            style={'textAlign': 'center',
                                'color': 'white'}),
                    html.P(f"{10}",
                            style={'textAlign': 'center',
                                'color': 'orange',
                                'fontSize': 40}),
                    ], className='card_container', style={'width': '16%', 'float': 'left'}),
                html.Div([
                    html.H6(children='Adjectives',
                            style={'textAlign': 'center',
                                'color': 'white'}),
                    html.P(f"{200}",
                            style={'textAlign': 'center',
                                'color': 'orange',
                                'fontSize': 40}),
                    ], className='card_container', style={'width': '16%', 'float': 'left'}),
                html.Div([
                    html.H6(children='Adverbs',
                            style={'textAlign': 'center',
                                'color': 'white'}),
                    html.P(f"{300}",
                            style={'textAlign': 'center',
                                'color': 'orange',
                                'fontSize': 40}),
                    ], className='card_container', style={'width': '16%', 'float': 'left'}),
                html.Div([
                    html.H6(children='Named Entities',
                            style={'textAlign': 'center',
                                'color': 'white'}),
                    html.P(f"{300}",
                            style={'textAlign': 'center',
                                'color': 'orange',
                                'fontSize': 40}),
                    ], className='card_container', style={'width': '16%', 'float': 'left'}),
                    
            ], className='row flex display'),
            html.Div([
                    html.H2(children='Nouns', style={'textAlign': 'center', 'color': 'white'}),
                    dcc.Graph(figure=fig_noun_cloud)
                ]),
            html.Div([
                    html.H2(children='Verbs', style={'textAlign': 'center', 'color': 'white'}),
                    dcc.Graph(figure=fig_verb_cloud)
                ]),
            html.Div([
                    html.H2(children='Adjectives', style={'textAlign': 'center', 'color': 'white'}),
                    dcc.Graph(figure=fig_adj_cloud)
                ])
        ])
    elif tab == 'tab-3':
        return html.Div([
            html.Div([
                    html.H4(f'MWEs (NC)'),
                        dash_table.DataTable(
                            id='table',
                            columns=[{"name": i, "id": i} for i in ['MWE', 'AM']],
                            data=tabledata_ncs,
                            # style_cell=dict(textAlign='left'),
                            style_header={'backgroundColor':'paleturquoise', 'textAlign':'center', 'font_family': 'san-serif', 'fontSize':20},
                            style_data={'backgroundColor':'aliceblue', 'textAlign':'left', 'font_family': 'san-serif', 'fontSize':18},
                            style_table={'height': 500,'overflowY': 'scroll'}
                        )
                ],style={'width': '50%', 'float': 'left'}),
            html.Div([
                    html.H4(f'MWEs (JNC)'),
                        dash_table.DataTable(
                            id='table',
                            columns=[{"name": i, "id": i} for i in ['MWE', 'AM']],
                            data=tabledata_jnc,
                            # style_cell=dict(textAlign='left'),
                            style_header={'backgroundColor':'paleturquoise', 'textAlign':'center', 'font_family': 'san-serif', 'fontSize':20},
                            style_data={'backgroundColor':'aliceblue', 'textAlign':'left', 'font_family': 'san-serif', 'fontSize':18},
                            style_table={'height': 500,'overflowY': 'scroll'}
                        )
                ],style={'width': '50%', 'float': 'left'}),
            
            ])
    elif tab == 'tab-4':
        return html.Div([
            html.H4(children='',
                    style={'textAlign': 'left',
                    'color': 'white'}),
            html.H4(children='Label Analysis from snlp',
                    style={'textAlign': 'left',
                    'color': 'white'}),
            html.H4(children='Include Buttons from Tutorial 3',
                    style={'textAlign': 'left',
                    'color': 'white'}),
            html.H4(children='Label Evaluation from Hawk',
                    style={'textAlign': 'left',
                    'color': 'white'}),
            html.Div([
                    html.H4(f'Label Analysis'),
                    dcc.Graph(figure=labels_figure)
                ]),
            ])
    elif tab == 'tab-4':
        return html.Div([
            html.H4(children='',
                    style={'textAlign': 'left',
                    'color': 'white'}),
            html.H4(children='Create a filterset',
                    style={'textAlign': 'left',
                    'color': 'white'}),
            html.H4(children='Replace MWEs in the corpus',
                    style={'textAlign': 'left',
                    'color': 'white'}),
            html.H4(children='Topic Extraction?',
                    style={'textAlign': 'left',
                    'color': 'white'}),
            ])

@app.callback(Output('dd-content', 'children'),
              [Input('select_option', 'value')]
              )
def show_country(dist):
    dis_name = 'Word Frequency' if dist=='wf' else ('Document Length' if dist=='dl' else 'Unknown')
    if dist == "wf":
        return html.Div([
                    html.H6(f'Showing the Distribution of: {dis_name}'),
                    dcc.Graph(figure=fig_w_freq)
                ])
    elif dist == "dl":
        return html.Div([
                    html.H6(f'Showing the Distribution of: {dis_name}'),
                    dcc.Graph(figure=doc_len_dist)
                ])

if __name__ == '__main__':
    app.run_server(debug=True)


import string
from typing import List, Tuple
import nltk
import time
from statistics import median
import os
import plotly
import pandas
import plotly.figure_factory as ff
import plotly.graph_objs as go
import pandas as pd
import numpy as np
import fasttext

from wordcloud import WordCloud, get_single_color_func
from wordview import logger
from tqdm import tqdm
from nltk.corpus import stopwords
from plotly.subplots import make_subplots

here = os.path.dirname(os.path.abspath(__file__))
ftmodel = fasttext.load_model(os.path.join(here, 'lid.176.ftz'))

# def create_adjust_subplots(labels: List[Tuple]) -> plotly.graph_objects.Figure:
#     """Create subplots and adjust the location of the titles wrt the number of labels.

#     Args:
#         labels (list): List of (label, type) tuples.

#     Returns:
#         fig (plotly.graph_objs.Figure)
#     """
#     if len(labels) == 0:
#         titles = (
#             "Analysis of Text",
#             "Analysis of Labels",
#             "Document Lengths",
#             "",
#             "",
#             "",
#             "",
#             "Word Frequency",
#             "",
#             "",
#             "",
#             "",
#             "" "Common Nouns",
#             "",
#             "",
#             "",
#             "Common Adjectives",
#             "",
#             "",
#             "",
#             "Common Verbs",
#         )
#     elif len(labels) == 1:
#         titles = (
#             "Analysis of Text",
#             "Analysis of Labels",
#             "Document Lengths",
#             labels[0][0].capitalize(),
#             "",
#             "",
#             "Word Frequency",
#             "",
#             "",
#             "",
#             "",
#             "Common Nouns",
#             "",
#             "",
#             "",
#             "Common Adjectives",
#             "",
#             "",
#             "",
#             "Common Verbs",
#         )
#     elif len(labels) == 2:
#         titles = (
#             "Analysis of Text",
#             "Analysis of Labels",
#             "Document Lengths",
#             labels[0][0].capitalize(),
#             "",
#             labels[1][0].capitalize(),
#             "Word Frequency",
#             "",
#             "",
#             "Common Nouns",
#             "",
#             "",
#             "",
#             "Common Adjectives",
#             "",
#             "",
#             "",
#             "Common Verbs",
#         )
#     elif len(labels) == 3:
#         titles = (
#             "Analysis of Text",
#             "Analysis of Labels",
#             "Document Lengths",
#             labels[0][0].capitalize(),
#             "",
#             labels[1][0].capitalize(),
#             "Word Frequency",
#             labels[2][0].capitalize(),
#             "",
#             "Common Nouns",
#             "",
#             "",
#             "",
#             "Common Adjectives",
#             "",
#             "",
#             "",
#             "Common Verbs",
#         )
#     elif len(labels) == 4:
#         titles = (
#             "Analysis of Text",
#             "Analysis of Labels",
#             "Document Lengths",
#             labels[0][0].capitalize(),
#             "",
#             labels[1][0].capitalize(),
#             "Word Frequency",
#             labels[2][0].capitalize(),
#             "",
#             "Common Nouns",
#             labels[3][0].capitalize(),
#             "",
#             "Common Adjectives",
#             "",
#             "",
#             "",
#             "Common Verbs",
#         )
#     fig = make_subplots(
#         rows=16,
#         cols=2,
#         subplot_titles=titles,
#         specs=[
#             [{}, {}],
#             [{"rowspan": 2}, {"rowspan": 2} if len(labels) >= 1 else {}],  # row 2
#             [None, None if len(labels) >= 1 else {}],
#             [{}, {"rowspan": 2} if len(labels) >= 2 else {}],  # row 4
#             [{"rowspan": 2}, None if len(labels) >= 2 else {}],
#             [None, {"rowspan": 2} if len(labels) >= 3 else {}],  # row 6
#             [{}, None if len(labels) >= 3 else {}],
#             [{"rowspan": 3}, {"rowspan": 2} if len(labels) >= 4 else {}],  # row 8
#             [None, None if len(labels) >= 4 else {}],
#             [None, {}],
#             [{"rowspan": 3}, {}],  # 12
#             [None, {}],
#             [None, {}],
#             [{"rowspan": 3}, {}],  # 16
#             [None, {}],
#             [None, {}],
#         ],
#         vertical_spacing=0.035,
#     )
#     return fig


# def generate_text_plots(
#     figure: plotly.graph_objs.Figure,
#     doc_length_list: List,
#     word_freq_list: List,
#     noun_freq_dict: dict,
#     adj_freq_dic: dict,
#     verb_freq_dic: dict,
# ) -> None:
#     """Generate distribution plots and word clouds for the textual content based on the input argumnets.

#     Args:
#         figure (plotly.graph_objs.Figure): Figure object in which the plots are created.
#         doc_length_list (list): List containing the length of each document.
#         word_freq_list (list): List containing the frequency of each word.
#         noun_freq_dict (dictionary): Dictionary of noun to frequency.
#         adj_freq_dict (dictionary): Dictionary of noun to frequency.
#         verb_freq_dict (dictionary): Dictionary of verb to frequency.

#     Returns:
#         None
#     """
#     # Customize plots and x, y labels: https://plotly.com/python/subplots/#customizing-subplot-axes
#     # Word cloud with plotly: https://github.com/PrashantSaikia/Wordcloud-in-Plotly/blob/master/plotly_wordcloud.py
#     def _distplot_to_dist_trace(data: List, color: str) -> dict:
#         """Create a trace from data.
#         Args:
#             data (list): List of numbers
#             color (list[str])

#         Returns:
#             data_dist (dict): Representation of a distplot figure.
#         """
#         data_dist = ff.create_distplot([data], group_labels=["distplot"], colors=[color])["data"]
#         for item in data_dist:
#             item.pop("xaxis", None)
#             item.pop("yaxis", None)
#         return data_dist

#     data1_dist = _distplot_to_dist_trace(doc_length_list, color="rgb(0, 200, 200)")
#     data2_dist = _distplot_to_dist_trace(word_freq_list, color="magenta")

#     d1_hist = data1_dist[0]
#     d1_kde = data1_dist[1]
#     d1_rug = data1_dist[2]

#     d2_hist = data2_dist[0]
#     d2_kde = data2_dist[1]
#     d2_rug = data2_dist[2]

#     noun_cloud = plotly_wordcloud(token_count_dic=noun_freq_dict)
#     adj_cloud = plotly_wordcloud(token_count_dic=adj_freq_dic)
#     verb_cloud = plotly_wordcloud(token_count_dic=verb_freq_dic)

#     figure.append_trace(d1_hist, 2, 1)
#     figure.append_trace(d1_kde, 2, 1)
#     figure.append_trace(d1_rug, 4, 1)

#     figure.append_trace(d2_hist, 5, 1)
#     figure.append_trace(d2_kde, 5, 1)
#     figure.append_trace(d2_rug, 7, 1)

#     figure.append_trace(noun_cloud, 8, 1)
#     figure.append_trace(adj_cloud, 11, 1)
#     figure.append_trace(verb_cloud, 14, 1)

#     figure.update_xaxes(rangemode="tozero", row=4, col=1)
#     figure.update_yaxes(showticklabels=False, row=4, col=1)

#     figure.update_xaxes(rangemode="tozero", row=7, col=1)
#     figure.update_yaxes(showticklabels=False, row=7, col=1)

#     figure.update_yaxes(title_text="Probability", row=2, col=1)
#     figure.update_yaxes(title_text="Probability", row=5, col=1)

#     figure.update_xaxes(showticklabels=False, zeroline=False, row=8, col=1)
#     figure.update_xaxes(showticklabels=False, zeroline=False, row=11, col=1)
#     figure.update_xaxes(showticklabels=False, zeroline=False, row=14, col=1)
#     figure.update_yaxes(showticklabels=False, zeroline=False, row=8, col=1)
#     figure.update_yaxes(showticklabels=False, zeroline=False, row=11, col=1)
#     figure.update_yaxes(showticklabels=False, zeroline=False, row=14, col=1)


def plotly_wordcloud(token_count_dic: dict) -> plotly.graph_objects.Scattergl:
    """Create a world cloud trace for plotly.

    Args:
        token_count_dic (dictionary): Dictionary of token to its count

    Returns:
        trace (plotly.graph_objects.Scatter)
    """
    wc = WordCloud(color_func=get_single_color_func("deepskyblue"), max_words=100)
    wc.generate_from_frequencies(token_count_dic)
    word_list = []
    rel_freq_list = []
    freq_list = []
    fontsize_list = []
    position_list = []
    orientation_list = []
    color_list = []
    for (word, rel_freq), fontsize, position, orientation, color in wc.layout_:
        word_list.append(word)
        rel_freq_list.append(rel_freq)
        freq_list.append(token_count_dic[word])
        fontsize_list.append(fontsize)
        position_list.append(position)
        orientation_list.append(orientation)
        color_list.append(color)
    # get the positions
    x = []
    y = []
    for i in position_list:
        x.append(i[0])
        y.append(i[1])
    # get the relative occurrence frequencies
    new_freq_list = []
    for i in rel_freq_list:
        i_tmp = round(i*100, 4)
        i_tmp = i_tmp if i_tmp > 1 else 1 # Plotly textfont.size in go.Scatter throws exception for values below 1.
        new_freq_list.append(i_tmp)
    try:
        trace = go.Scattergl(
            x=x,
            y=y,
            textfont=dict(size=new_freq_list, color=color_list),
            hoverinfo="text",
            hovertext=["{0}: {1}".format(w, f) for w, f in zip(word_list, freq_list)],
            mode="text",
            text=word_list,
        )
        return trace
    except Exception as E:
        logger.error(f'While creating the word cloud, plotly.go returned the following error \
         \n{E}\nfor relative frequencies: {rel_freq_list}\nthat were mapped to {new_freq_list}')


def generate_label_plots(df: pandas.DataFrame, label_cols: List[Tuple]) -> None:
    """Generate histogram and bar plots for the labels in label_cols.

    Args:
        figure (plotly.graph_objs.Figure): Figure object in which the plots are created.
        df (Pandas DataFrame): DataFrame that contains labels specified in label_cols.
        label_cols (list): list of tuples in the form of [('label_1', 'categorical/numerical'),
                           ('label_2', 'categorical/numerical'), ...]

    Returns:
        None
    """
    
    if len(label_cols) == 1:
        figure = make_subplots(rows=1, cols=1,subplot_titles=("Plot 1"))
        lab_trace1 = label_plot(df, label_col=label_cols[0][0], label_type=label_cols[0][1])
        figure.append_trace(lab_trace1, 1, 1)
        figure.update_yaxes(title_text="Count", row=2, col=2)
    elif len(label_cols) == 2:
        figure = make_subplots(rows=1, cols=2,subplot_titles=("Plot 1", "Plot 2"))
        lab_trace1 = label_plot(df, label_col=label_cols[0][0], label_type=label_cols[0][1])
        lab_trace2 = label_plot(df, label_col=label_cols[1][0], label_type=label_cols[1][1])
        figure.append_trace(lab_trace1, 1, 1)
        figure.append_trace(lab_trace2, 1, 2)
        # figure.update_yaxes(title_text="Count", row=2, col=2)
        # figure.update_yaxes(title_text="Count", row=4, col=2)
    elif len(label_cols) == 3:
        figure = make_subplots(rows=1, cols=3,subplot_titles=("Plot 1", "Plot 2", "Plot 3"))
        lab_trace1 = label_plot(df, label_col=label_cols[0][0], label_type=label_cols[0][1])
        lab_trace2 = label_plot(df, label_col=label_cols[1][0], label_type=label_cols[1][1])
        lab_trace3 = label_plot(df, label_col=label_cols[2][0], label_type=label_cols[2][1])
        figure.append_trace(lab_trace1, 1, 1)
        figure.append_trace(lab_trace2, 1, 2)
        figure.append_trace(lab_trace3, 1, 3)
        # figure.update_yaxes(title_text="Count", row=2, col=2)
        # figure.update_yaxes(title_text="Count", row=4, col=2)
        # figure.update_yaxes(title_text="Count", row=6, col=2)
    elif len(label_cols) == 4:
        figure = make_subplots(rows=2, cols=2,subplot_titles=("Plot 1", "Plot 2", "Plot 3", "Plot 4"))
        lab_trace1 = label_plot(df, label_col=label_cols[0][0], label_type=label_cols[0][1])
        lab_trace2 = label_plot(df, label_col=label_cols[1][0], label_type=label_cols[1][1])
        lab_trace3 = label_plot(df, label_col=label_cols[2][0], label_type=label_cols[2][1])
        lab_trace4 = label_plot(df, label_col=label_cols[3][0], label_type=label_cols[3][1])
        figure.append_trace(lab_trace1, 1, 1)
        figure.append_trace(lab_trace2, 1, 2)
        figure.append_trace(lab_trace3, 2, 1)
        figure.append_trace(lab_trace4, 2, 2)
        # figure.update_yaxes(title_text="Count", row=2, col=2)
        # figure.update_yaxes(title_text="Count", row=4, col=2)
        # figure.update_yaxes(title_text="Count", row=6, col=2)
        # figure.update_yaxes(title_text="Count", row=8, col=2)
    return figure


def label_plot(df: pandas.DataFrame, label_col: str, label_type: str) -> plotly.graph_objects.Histogram:
    """Create a plot for label_col in df, wrt to label_type.

    Args:
        df (Pandas DataFrame): DataFrame that contains label_col.
        label_col (str): Name of the label column in df that must be plotted.
        label_type (str): Represents the type of label and consequently specifies the type of plot.
                             It can be "numerical" or "categorical".

    Returns:
        trace (plotly.graph_objects.Histogram)
    """
    if label_type == "categorical":
        values = df[label_col].unique().tolist()  # ['pos', 'neg', 'neutral']
        counts = df[label_col].value_counts()  # 1212323
        x = []
        y = []
        for v in values:
            x.append(v)
            y.append(counts[v])
        trace = go.Bar(x=x, y=y, name=label_col)
    elif label_type == "numerical":
        trace = go.Histogram(x=df[label_col], name=label_col)
    else:
        raise ValueError('label_col input argument must be set to either "categorical" or "numerical".')
    return trace


def do_analysis(
    df: pandas.DataFrame,
    out_dir: str,
    text_col: str,
    label_cols: List = [],
    language: str = "english",
    skip_stopwords_punc: bool = True,
    save_report: bool = False,
) -> None:
    """Generate analysis report and eitherr renders the report via Plotly show api or saves it offline to html.

    Args:
        df (pandas.DataFrame): DataFrame that contains text and labels.
        out_dir (str): Dir where the report is saved. Required only when save_report option is True.
        text_col (str): Name of the column that contains a tokenized text content.
        label_cols (list): list of tuples in the form of [('label_1', 'categorical/numerical'),
                           ('label_2', 'categorical/numerical'), ...]
        language (str): Language of the text in df[text_col]
        skip_stopwords_punc (bool): Whether or not skip stopwords and punctuations in the analysis. Default: True
        save_report (bool): Whether or not save the report as an html file. Default: False

    Returns:
        None
    """

    global ftmodel

    def update_count(items_dic: dict, items: List[str]) -> None:
        """Update the corresponding count for each key in  items_dic. w.r.t. terms in items.

        Args:
            items_dic (dict): Dictionary mapping keys to their count
            items (list): List of tokens

        Returns:
            None
        """
        for t in items:
            if t in items_dic:
                items_dic[t] += 1
            else:
                items_dic[t] = 1

    def get_pos(tagged_tokens: List[Tuple[str, str]], goal_pos: str) -> List:
        """Extracts goal_pos POS tags from tagged_tokens.

        Args:
            tagged_tokens (List[Tuple(str, str)]): Contains terms and ther pos tags. E.g.
                                                   [('cat', 'NN'), ('sat', 'VBD'), ('on', 'IN'), ('mat', 'NN')]
            goal_pos (str): Pos tag to look for in tagged_tokens

        Returns:
            res (List(str)): List of tokens with goal_pos pos tag
        """
        res = []
        for pt in tagged_tokens:
            if pt[1].startswith(goal_pos):
                res.append(pt[0])
        return res

    if len(label_cols) > 4:
        raise ValueError("Maximum of 4 labels can be specified for analysis.")

    stop_words = set(stopwords.words(language))
    punctuations = set(string.punctuation)

    doc_lengths = []
    token_to_count = {}
    NNs = {}
    JJs = {}
    Vs = {}
    languages = set()

    logger.info("Processing text in %s column of the input DataFrame..." % text_col)
    for text in tqdm(df[text_col]):
        ls = ftmodel.predict(text)[0][0].replace('__label__', '').upper()
        languages.update([ls])
        try:
            tokens = text.lower().split(" ")
            doc_lengths.append(len(tokens))
            if skip_stopwords_punc:
                tokens = [t for t in tokens if t not in stop_words and t not in punctuations]
                update_count(token_to_count, tokens)

        except Exception as e:
            logger.warning("Processing entry --- %s --- lead to exception: %s" % (text, e.args[0]))
            continue

        postag_tokens = nltk.pos_tag(tokens)
        nouns = get_pos(postag_tokens, "NN")
        update_count(NNs, nouns)
        verbs = get_pos(postag_tokens, "VB")
        update_count(Vs, verbs)
        adjectives = get_pos(postag_tokens, "JJ")
        update_count(JJs, adjectives)
    
    freq_df = pd.DataFrame({'tokens': token_to_count.keys(), 'count':token_to_count.values()})
    freq_df['proportion'] = freq_df['count']/freq_df['count'].sum()
    vocab_size = freq_df.shape[0]
    n_tokens = freq_df['count'].sum()
    s = 1
    denom = np.sum(1/(np.arange(1, n_tokens+1)**s))

    def classic_zipf(k, s=s, denom=denom):
        num = 1/k**s
        res = num/denom
        return res 

    logger.info("Calculating Empirical and Theoretical Zipf values...")
    freq_df["position"] = freq_df.index+1
    start = time.time()
    freq_df['predicted_proportion'] = freq_df["position"].apply(classic_zipf)
    end = time.time()
    logger.info(f'Time to measure predicted proportion for {freq_df.shape[0]} rows: {end - start}')

    x = np.log(freq_df["position"].values)
    y_emperical = np.log(freq_df['count'])
    y_theoritical = np.log(freq_df['predicted_proportion'] * n_tokens)



    # fig_main = create_adjust_subplots(label_cols)
    # logger.info("Generating distplots and word cloud for input text")
    # generate_text_plots(fig_main, doc_lengths, word_frequencies, NNs, JJs, Vs)
    # logger.info("Generating plots for labels")
    # generate_label_plots(fig_main, df, label_cols)
    # logger.info("Rendering plots")
    # fig_main.update_layout(height=3100, showlegend=False)
    # if save_report:
    #     plotly.offline.plot(fig_main, filename=os.path.join(out_dir, "report.html"))
    # else:
    #     fig_main.show()
    
    return TextAnalysisResult(doc_lengths=doc_lengths,
                            zipf_x=x,
                            zipf_y_emp=y_emperical,
                            zipf_y_theory=y_theoritical,
                            languages=languages,
                            type_count=vocab_size,
                            token_count=n_tokens,
                            doc_count=len(doc_lengths),
                            median_doc_len=median(doc_lengths),
                            nns=NNs,
                            jjs=JJs,
                            vs=Vs)


class TextAnalysisResult():
   def __init__(self, doc_lengths, zipf_x, zipf_y_emp, zipf_y_theory, languages, type_count, token_count, doc_count, median_doc_len, nns, jjs, vs):
        self.doc_lengths = doc_lengths
        self.zipf_x = zipf_x
        self.zipf_y_emp = zipf_y_emp
        self.zipf_y_theory = zipf_y_theory
        self.languages = languages
        self.type_count = type_count
        self.token_count = token_count
        self.doc_count = doc_count 
        self.median_doc_len = median_doc_len
        self.nns = nns
        self.jjs = jjs
        self.vs = vs


# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['wordview',
 'wordview.dashapp',
 'wordview.evaluation',
 'wordview.mwes',
 'wordview.preprocessing',
 'wordview.text_analysis']

package_data = \
{'': ['*'], 'wordview.dashapp': ['assets/*']}

install_requires = \
['dash==2.0.0',
 'fasttext==0.9.2',
 'nltk==3.6.6',
 'pandas==1.3.5',
 'plotly==5.5.0',
 'scikit-learn==1.0.1',
 'scipy==1.7.3',
 'tqdm==4.62.3',
 'wordcloud==1.8.1']

entry_points = \
{'console_scripts': ['script_download = wordview.bin.downloads:download_req']}

setup_kwargs = {
    'name': 'wordview',
    'version': '0.1.0',
    'description': 'WORDVIEW is a Python package for text analysis.',
    'long_description': '# wordview\n\n[![PyPI version](https://badge.fury.io/py/wordview.svg?&kill_cache=1)](https://badge.fury.io/py/wordview)\n\n[![Python 3.7](https://img.shields.io/badge/python-3.7-blue.svg)](https://www.python.org/downloads/release/python-370/)\n\n\nWORDVIEW is a Python package for primarily text analysis. It, moreover, provides a number of unsupervised models for Information Extraction and Preprocessing. See section [Features](#Features) for different functionalities.\n\nWORDVIEW is open-source and free. We, however, developed a Dashboard version of WORDVIEW based on Plotly, for non-developers. See WORDVIEW-DASH page here.\n\n\n# Features\n- [Text Analysis](#text-analysis)\n- Information Extraction\n  - [Extraction of Multiword Expressions (statistically idiomatic aka collocations)](#extraction-of-multiword-expressions)\n  - Extraction of Non-compositional Multiword Expressions (semantically idiomatic)\n  - Anomalies\n- [Text Cleaning](#text-cleaning)\n  - [Identification and filtering of Statistically Redundant Words](#identification-of-statistically-redundant-words)\n  - [Auto Text Cleaning](#auto-text-cleaning)\n\n## Upcoming Features\n- **Identification of Non-compositional Expressions (e.g. *red tape* and *brain drain*)**. Non-compositional expressions have application in profanity detection, language understanding, and language generation.\n\n- **Entropy for Natural Language** Entropy has a wide range of applications in NLP. Recently, researchers have show that it can be used to improve the quality of conversational AI [[1]](#1) and text summarization [[2]](#2).\n\n- **Label Evaluation** Labeled datasets often come with certain level of human error. If not captured early on, this these errors will propagate to downstream machine learning models and hinder their quality rendering an otherwise well-performing model performs poorly. At model building and training time there is no easy way to identify if the error lies in the labels or the model itself. This often leads to spending a large amount of time trying to fix the problem. To avoid this, Label Evaluation features provides a set of functionalities to evaluate the labels and identify problematic ones based on measures of Inter Rater Agreement, and Correlations.\n\n\n# Usage\n\nInstall the package:\n\n`pip install wordview`\n\nSee the description of different functionalities with worked examples below. \nAt first, let\'s load some test data. Note the format of the input data. It is a `csv` file comprising `text` and `label` columns:\n\n```python\nimport pandas as pd\nimdb_train = pd.read_csv(\'data/imdb_train_sample.tsv\', sep=\'\\t\', names=[\'label\', \'text\'])\n\nimdb_train.head()\n\n  label                                               text\n0   neg  well , i rented this movie and found out it re...\n1   pos  you know , this movie is n\'t that great , but ...\n2   pos  a heartwarming film . the usual superb acting ...\n3   pos  i did n\'t expect to like this film as much as ...\n4   pos  i could n\'t help but feel that this could have...\n```\n\n\n## **Text Analysis**\n\n*wordview* provides an easy to use function (`text_analysis.generate_report`) for analyzing text with an extensive analysis report. `text_analysis.generate_report` \nreceives as input a Pandas Dataframe that contains a text column, and an optional number of label columns. Currently, `text_analysis.generate_report` can generate plots for up to 4 numerical or categorical labels. See the example below for more details. \n\n```python\nfrom wordview.text_analysis import generate_report\n\ngenerate_report(df=imdb_train,\n                out_dir=\'output_dir\',\n                text_col=\'text\',\n                label_cols=[(\'label\', \'categorical\')])\n\n```\n\nThe above script creates an analysis report that includes distribution plots and word clouds for different POS tags, for text, and bar plots and histograms for labels. You can specify up to \n4 labels of type *categorical* or *numerical*. See the example below for including another label of *numerical* type. The report is automatically rendered in the browser via `plotly` port assignment. But you also have the option of saving the report in the HTML format by setting the `save_report` argument to `True`. \n\n```python\nimport numpy as np\nimport random\n\n# In addition to the original label, for illustration purpose, let\'s create two random labels:\nimdb_train[\'numerical_label\'] = np.random.randint(1, 500, imdb_train.shape[0])\nimdb_train[\'new_label\'] = random.choices([\'a\', \'b\', \'c\', \'d\'], [0.2, 0.5, 0.8, 0.9], k=imdb_train.shape[0])\n\ngenerate_report(df=imdb_train,\n                out_dir=\'output_dir\',\n                text_col=\'text\',\n                label_cols=[(\'label\', \'categorical\'), (\'new_label\', \'categorical\'), (\'numerical_label\', \'numerical\')])\n\n```\n\nThe above yields a report in HTML, with interactive `plotly` plots as can be seen in example screenshots below. \n\n![annotation1](/figs/annotation1.png)\n\n You can easily zoom in any part of the plot to a have a closer look:\n\n![zoom](/figs/zoom.png)\n\nYou can get word clouds for different part of speech tags, as can be seen in the below example where word clouds for nouns, adjectives and verbs are rendered:\n\n![wc](/figs/wc.png)\n\n## **Extraction of Multiword Expressions**\n\nMultiword Expressions (also known as collocations of fixed expressions) are phrases that function as a single semantic unit E.g. *swimming pool* and *climate change*. Multiword Expressions have application in a wide range of NLP tasks ranging from sentiment analysis to topic models and key-phrase extraction. \n\nYou can use `wordview` to identify different types of MWEs in your text leveraging statistical measures such as *PMI* and *NPMI*. To do so, first create an instance of `MWE` class:\n\n\n```python\nfrom wordview.mwes import MWE\nmy_mwe_types = ["NC", "JNC"]\nmwe = MWE(df=imdb_train, mwe_types=my_mwe_types, text_column=\'text\')\n```\n\nIf the text in `text_column` is un-tokenized or poorly tokenized, `MWE` recognizes this issue at instantiation time and shows you a warning. If you already know that your text is not tokenized, you can run the same instantiation with flag `tokenize=True`. Next you need to run the method `build_count()`. Since creating counts is a time consuming procedure, it was implemented independently from `extract_mwes()` method that works on top of the output of `build_count()`. This way, you can get the counts which is a time consuming process once, and then run `extract_mwes()` several times with different parameters.\n\n```python\nmwe.build_counts()\nmwe.extract_mwes()\n```\n\nRunning the above results in a json file, containing dictionary of mwe types defined in the `mwe_types` argument of `MWE`, to their association score (specified by `am` argument of `extract_mwes()`). Note that the MWEs in this json file are sorted with respect to their `am` score. All MWEs and their counts are stored in respective directories inside the `output_dir` argument of `MWE`. The default value is `tmp`. \n\n```\nNOUN-NOUN COMPOUNDS\n-------------------\njet li\nclint eastwood\nmonty python\nkung fu\nblade runner\n\n\nADJECTIVE-NOUN COMPOUNDS\n------------------------\nspinal tap\nmartial arts\ncitizen kane\nfacial expressions\nglobal warming\n```\n\nAn important use of extracting MWEs is to treat them as a single token. Research shows that when fixed expressions are treated as a single token rather than the sum of their components, they can improve the performance of downstream applications such as classification and NER. Using the `replace_mwes` function, you can replace the extracted expressions in the corpus with their hyphenated version (global warming --> global-warming) so that they are considered a single token by downstream applications. A worked example can be seen below:\n\n```python\nfrom wordview.mwes import replace_mwes\nnew_df = replace_mwes(path_to_mwes=\'tmp/mwes/mwe_data.json\', mwe_types=[\'NC\', \'JNC\'], df=imdb_train, text_column=\'text\')\nnew_df.to_csv(\'tmp/new_df.csv\', sep=\'\\t\')\n```\n\n\n## **Identification of Statistically Redundant Words**\n\nRedundant words carry little value and can exacerbate the results of many NLP tasks. To solve this issue, traditionally, a pre-defined list of words, called stop words was defined and removed from the data. However, creating such a list is not optimal because in addition to being a rule-based and manual approach which does not generalize well, one has to assume that there is a universal list of stop words that represents highly low entropy words for all corpora, which is a very strong assumption and not necessarily a true assumption in many cases.\n\nTo solve this issue, one can use a purely statistical solution which is completely automatic and does not make any universal assumption. It focuses only on the corpus at hand. Words can be represented with various statistics. For instance, they can be represented by their term frequency (tf) or inverse document frequency (idf). It can be then interpreted that terms with anomalous (very high or very low) statistics carry little value and can be discarded.\nWORDVIEW enables you to identify such terms in an automatic fashion. The solution might seem complex behind the scene, as it firsts needs calculate certain statistics, gaussanize the distribution of the specified statistics (i.e. tf or ifd), and then identify the terms with anomalous values on the gaussanized distribution by looking at their z-score. However, the API is easy and convenient to use. The example below shows how you can use this API:\n\n```python\nfrom wordview.preprocessing import RedunWords\n\nrw = RedunWords(imdb_train["text"], method=\'idf\')\n```\n\nLet the program automatically identify a set of redundant words:\n\n```python\nred_words = rw.get_redundant_terms()\n```\n\n\nAlternatively, you can manually set cut-off threshold for the specified score, by setting the manual Flag to True and specifying lower and upper cut-off thresholds. \n```python\nred_words = rw.get_redundant_terms(manual=True, manual_thresholds: dict={\'lower_threshold\':1, \'upper_threshold\': 8})\n```\n\nIn order to get a better understanding of the distribution of the scores before setting the thresholds, you can run `show_plot()` method from `RedunWords` class to see this distribution:\n\n```python\nrw.show_plot()\n```\n\nWhen red_words is ready, you can filter the corpus:\n\n```python\n# text must be a list of words\nres = " ".join([t for t in text if t not in redundant_terms])\n```\n\n## **Text Cleaning**\n\n*wordview* implements an easy to use and powerful function for cleaning up the text (`clean_text`). \nUsing, `clean_text`, you can choose what pattern to accept via `keep_pattern` argument, \nwhat pattern to drop via `drop_patterns` argument, and what pattern to replace via `replace` argument. You can also specify the maximum length of tokens. \nLet\'s use [Stanford\'s IMDB Sentiment Dataset](https://ai.stanford.edu/~amaas/data/sentiment/) as an example. A sample of this data can be found in `resources/data/imdb_train_sample.tsv`.\n\n\n```python\nfrom wordview.preprocessing import clean_text\n\nimdb_train = pd.read_csv(\'data/imdb_train_sample.tsv\', sep=\'\\t\', names=[\'label\', \'text\'])\n\n# Let\'s only keep alphanumeric tokens as well as important punctuation marks:\nkeep_pattern=\'^[a-zA-Z0-9!.,?\\\';:$/_-]+$\'\n\n# In this corpus, one can frequently see HTML tags such as `< br / >`. So let\'s drop them:\ndrop_patterns={\'< br / >\'}\n\n# By skimming throw the text one can frequently see many patterns such as !!! or ???. Let\'s replace them:\nreplace={\'!!!\':\'!\', \'\\?\\?\\?\':\'?\'}\n\n# Finally, let\'s set the maximum length of a token to 15:\nmaxlen=15\n\n# Pass the set keyword arguments to the apply:\nimdb_train.text = imdb_train.text.apply(clean_text, args=(), keep_pattern=keep_pattern, replace=replace, maxlen=maxlen)\n```\n\nNote that `clean_text` returns tokenized text. \n\n\n# Contributions\n\n```bash\n# Create and activate a virtual env\npython -m venv VENV\nsource VENV/bin/activate\n# Install dependencies\npip install -r requirements.txt\n# Run app\npython wordview/dashapp/index.py\n```\n\n\n## References\n<a id="1">[1]</a> R. Csaky et al. - Improving Neural Conversational Models with Entropy-Based Data Filtering - In Proceedings of ACL 2019 - Florence, Italy.\n\n<a id="2">[2]</a> Maxime Peyrard - A Simple Theoretical Model of Importance for Summarization - In Proceedings of ACL 2019 - Florence, Italy.\n\n## **Auto Text Cleaning**\nOne of the first obstacles that any NLP practitioner faces is the tedious, demotivating, and confusing task of cleaning up the text. Unlike images that come in a perfect-for-ml vector format which is the same across the globe and input sources, text comes in all forms, formats, styles, languages, and structures. Before being able to get value out of it, any NLP expert has gone through the painful and tedious task of text cleaning. Even worse, there is no universal recipe for this. That is to say, should I lowercase this text? Shall I replace numbers with a place holder or should remove them altogether? How about stop words?... Each person can carry out a subset of the mentioned example steps, and arrive at a different cleaned text. How can we compare models then? A slight change in the often overlooked cleaning step can lead to inconsistency in our experiments. ',
    'author': 'meghdadFar',
    'author_email': 'meghdad.farahmand@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)
